// ***********************************************************
// Project: marder1 marder and rat chaser
// Author: Dr Simon Schroedle
// Started: 11/2015
// ***********************************************************
#define F_CPU 4000000UL

#include <avr/io.h>
#include <util/delay.h>


/*Initialize TIMER1*/
void init_frq(void)
{
  TCCR1A = 0b11000000; // Set OC1A when up counting, clear when down counting
  TCCR1B = 0b00010000; // Phase/Freq-correct PWM, top value = ICR1, not running at start
}

void set_frq(uint16_t top_value)
{
   uint16_t duty_cycle;

   //
   // TOP = Board Clock Frequency / (2 x N x Output Frequency)
   // Where N is Prescaler

   // Reset the TIMER1 16 bit Counter
   TCNT1H = 0;
   TCNT1L = 0;

   // Set the TIMER1 Counter TOP value on ICR1H and ICR1L
   ICR1H = (top_value >> 8 ) & 0x00FF;
   ICR1L = top_value;

   // Set the TIMER1 PWM Duty Cycle on OCR1AH and OCR1AL
   // Use half of the TOP value (PWM Duty cycle about 50%)

   duty_cycle=top_value / 2;
   OCR1AH=(duty_cycle >> 8 ) & 0x00FF;
   OCR1AL=duty_cycle;

   TCCR1A = 0b11000000;
   TCCR1B |= (1<<CS10);  //Prescaler 1; this switches the signal ON

   PORTB|=(1<<PB0); //Indicator LED: signal ON

}

void tone_off(void)
{
   TCCR1B &= ~(1<<CS10); //remove frequency source
   TCCR1A = 0b00000000; //switch off timer; otherwise, port can not be set low

   PORTB&=~(1<<PB1); //make sure output is low when counter not active
   PORTB&=~(1<<PB0); //LED status: OFF
}

uint8_t frequencies1[21]={111,100,83,77,71,
		100,111,71,77,83,
		77,67,100,111,91,
		111,77,83,91,67,
		91};//these values are for a 4 MHz Xtal, prescaler = 1


void delay10ms(uint16_t dm)
{
  uint16_t dcnt;
  for (dcnt=0;dcnt<dm;dcnt++)
  {
    _delay_loop_2(9999); //delay about 10 ms for 4 mhz clock
  }

}

uint16_t gdelays[13]={400,320,100,200,250,
                    50,200,123,434,213,
                    100,50,100};

uint16_t qdelays[8]={1400,2320,1100,500,2250,150,320,1200};
//these delays are in 10s of ms

int main(void)
{

   uint8_t gcnt,qcnt,fcnt;

   DDRB|=(1<<PB1);//output: PWM signal
   DDRB|=(1<<PB0);//output: LED status

   init_frq();

   while(1)
   {
      gcnt++;
      if (gcnt>12)
      {
        gcnt=0;
        qcnt++;
        if (qcnt>7)
        {
          qcnt=0;
        }
        tone_off();
        delay10ms(qdelays[qcnt]);
        init_frq();
      }
      fcnt++;
      if (fcnt>20)
      {
        fcnt=0;
      }
      set_frq(frequencies1[fcnt]);
      delay10ms(gdelays[gcnt]);

   }
}


